/*
	Copyright (c) 2004 Jonatan Kronqvist

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the "Software"),
	to deal in the Software without restriction, including without limitation
	the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
	IN THE SOFTWARE.
*/



#ifndef __FOO_BLUETOOTH_CTRL_H__
#define __FOO_BLUETOOTH_CTRL_H__

#include "../SDK/foobar2000.h"
#include "../helpers/helpers.h"

#include "resource.h"
#include "connection.h"
#include "macros.h"
#include "bluetooth_control.h"
#include "control.h"
#include "foobar_control.h"


#define WM_BT_CMD	0x8000



/**
 * config variables
 */
static cfg_string cfg_comport("bluetooth_ctrl_comport", "COM4");
static cfg_string cfg_musicdir("bluetooth_ctrl_musicdir", "H:\\mp3");
static cfg_string cfg_playlist_fmt("bluetooth_ctrl_playlist_fmt", "$if2(%artist% - %title%, %_filename_ext%)");
static cfg_int cfg_autoconnect("bluetooth_ctrl_autoconnect", 0);
static cfg_int cfg_random("bluetooth_ctrl_random", 0);
static cfg_int cfg_specific_playlist("bluetooth_ctrl_specific_playlist", 0);
static cfg_int cfg_selected_playlist("bluetooth_ctrl_selected_playlist", 0);


/**
 * Main class
 */
class foo_bluetooth_ctrl
{
public:
	foo_bluetooth_ctrl();
	~foo_bluetooth_ctrl();

	void set_active(bool activate);
	bool is_active();
	void read_command();
	void update_config();

private:
	bluetooth_control *m_bt_control;
	connection *m_conn;
	control *m_play_control;
	HANDLE	m_io_thread;
	HWND	m_hwnd;
	bool	m_activated;

	static unsigned long WINAPI read_command_thread(LPVOID param);
	static LRESULT CALLBACK wndproc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
};


foo_bluetooth_ctrl *g_bt_ctrl = NULL;



/**
 * So that we don't have to have a lot of ugly #ifdefs all around
 * the code. Easier to read this way..
 */
void g_debug(string8 str)
{
#ifdef _DEBUG
	console::info(str);
#endif
}




/**
 * Called on foobar init/quit
 */
class initquit_bluetooth : public initquit
{
public:
	virtual void on_init()
	{ 
		g_bt_ctrl = new foo_bluetooth_ctrl();	// create new instance
		if(cfg_autoconnect != 0)
			g_bt_ctrl->set_active(true);				// activate if autoconnect is set
	}
	virtual void on_quit()
	{
		g_bt_ctrl->set_active(false);
		SAFE_DELETE(g_bt_ctrl);
	}
};




/**
 * Components-Toggle Bluetooth Connection menu item
 */
class components_menu_item_bluetooth : public menu_item_main_single
{
public:
	void get_name(string_base &out)
	{
		out.set_string("Components/Bluetooth Connection");
	}
	void run()
	{
		g_bt_ctrl->set_active(!g_bt_ctrl->is_active());		// toggle active
	}
	bool is_checked()
	{
		return g_bt_ctrl->is_active();
	}
	virtual bool get_description(string_base & out) {return false;}
};




/**
 * Config menu (preferences)
 */
class config_bluetooth_ctrl : public config
{
	static BOOL CALLBACK ConfigProc(HWND wnd,UINT msg,WPARAM wp,LPARAM lp);
	HWND create(HWND parent)
	{
		return uCreateDialog(IDD_CONFIG_BLUETOOTH_CTRL,parent,ConfigProc);
	}
	const char * get_name() {return "Bluetooth Remote Control";}
	const char * get_parent_name() {return "Components";}
};




static service_factory_single_t<initquit, initquit_bluetooth> iqbt;
static service_factory_single_t<menu_item, components_menu_item_bluetooth> cmibt;
static service_factory_single_t<config, config_bluetooth_ctrl> cbtc;




DECLARE_COMPONENT_VERSION(
	"Bluetooth Remote Control",
	"1.03",
	"Copyright (c) 2004 Jonatan Kronqvist <jonatan@oxygenite.org>"
);


#endif /*__FOO_BLUETOOTH_CTRL_H__*/